#! /usr/bin/env python3

import itertools
import math
import os
import subprocess

from lab.environments import LocalEnvironment, BaselSlurmEnvironment
from lab.reports import Attribute, geometric_mean, arithmetic_mean

from downward.reports.compare import ComparativeReport

import common_setup
from common_setup import IssueConfig, IssueExperiment

DIR = os.path.dirname(os.path.abspath(__file__))
SCRIPT_NAME = os.path.splitext(os.path.basename(__file__))[0]
BENCHMARKS_DIR = f"{DIR}/../pddl/2022-04-07"
PLANNERS_DIR = f"{DIR}/../planners/"
ENVIRONMENT = BaselSlurmEnvironment(partition="rome",email="bx56lg@cs.aau.dk")

SEARCH_TIME_LIMIT = 60
OVERALL_TIME_LIMIT = 120

SUITE = ['emoplan-partialcontrol-feasible']

if common_setup.is_test_run():
    ENVIRONMENT = LocalEnvironment(processes=4)

exp = IssueExperiment(
    environment=ENVIRONMENT,
)

exp.add_algorithm("blind",
                  f"{PLANNERS_DIR}/downward",
                  "main",
                  ['--search',  'astar(blind)'],
                  build_options = ["-j1"],
                  driver_options = ["--search-time-limit", f"{SEARCH_TIME_LIMIT}", "--overall-time-limit", f"{OVERALL_TIME_LIMIT}", ])

exp.add_algorithm("lmcut",
                  f"{PLANNERS_DIR}/downward",
                  "main",
                  ['--search',  'astar(lmcut)'],
                  build_options = ["-j1"],
                  driver_options = ["--search-time-limit", f"{SEARCH_TIME_LIMIT}", "--overall-time-limit", f"{OVERALL_TIME_LIMIT}", ])

exp.add_algorithm("scorpion-sys3-30",
                  f"{PLANNERS_DIR}/downward",
                  "scorpion",
                  ['--search', "astar(scp_online([projections(sys_scp(max_pattern_size=3, max_pdb_size=2M, max_collection_size=20M, max_time=30, max_time_per_restart=infinity, saturate=False,  store_dead_ends=False, pattern_type=interesting_non_negative, max_patterns=infinity, order=cg_down)) , cartesian()], saturator=perimstar, max_time=50, interval=10K, orders=greedy_orders()), pruning=limited_pruning(pruning=atom_centric_stubborn_sets(), min_required_pruning_ratio=0.2))"],
                  build_options = ["-j1"],
                  driver_options = ["--transform-task", "preprocess-h2", "--search-time-limit", f"{SEARCH_TIME_LIMIT}", "--overall-time-limit", f"{OVERALL_TIME_LIMIT}", ])


exp.add_algorithm("gamer-interleaved",
                  f"{PLANNERS_DIR}/downward",
                  "lemmy-experiments",
                  ['--search', "astar_interleaved(modular_pdb(gamer(time_limit=5),open_list_evaluator(time_limit=2),local_search_gamer(time_limit=2),10,pdb_factory=modular_symbolic,create_perimeter=false,only_gamer=true,doing_local_search=true,doing_canonical_search=false))"],
                  build_options = ["-j1"],
                  driver_options = ["--search-time-limit", f"{SEARCH_TIME_LIMIT}", "--overall-time-limit", f"{OVERALL_TIME_LIMIT}", ])


exp.add_algorithm("complementary2",
                  f"{PLANNERS_DIR}/downward",
                  "ipc-2018-seq-opt",
                  ['--search', 'astar(cpdbs_symbolic(genetic_ss(use_ucb=true,num_episodes=10000000,num_collections=1,pdb_factory=symbolic,genetic_time_limit=1,time_limit=1.0,create_perimeter=true,use_first_goal_vars=true,use_norm_dist=true,perimeter_time_ms=1000,perimeter_step_time_ms=500,perimeter_nodes=100000)))'],
                  build_options = ["-j1"],
                  driver_options = ["--search-time-limit", f"{SEARCH_TIME_LIMIT}", "--overall-time-limit", f"{OVERALL_TIME_LIMIT}", ])

exp.add_algorithm("sbd",
                  f"{PLANNERS_DIR}/downward",
                  "symbolic",
                  ['--search',  'sbd'],
                  build_options = ["-j1"],
                  driver_options = ["--search-time-limit", f"{SEARCH_TIME_LIMIT}", "--overall-time-limit", f"{OVERALL_TIME_LIMIT}", ])

exp.add_algorithm("sfw",
                  f"{PLANNERS_DIR}/downward",
                  "symbolic",
                  ['--search',  'sfw'],
                  build_options = ["-j1"],
                  driver_options = ["--search-time-limit", f"{SEARCH_TIME_LIMIT}", "--overall-time-limit", f"{OVERALL_TIME_LIMIT}", ])


exp.add_algorithm("wastar-lmcut-2",
                  f"{PLANNERS_DIR}/downward",
                  "main",
                  ['--search',  "eager_wastar([lmcut], w=2)"],
                  build_options = ["-j1"],
                  driver_options = ["--search-time-limit", f"{SEARCH_TIME_LIMIT}", "--overall-time-limit", f"{OVERALL_TIME_LIMIT}", ])

exp.add_algorithm("ff",
                  f"{PLANNERS_DIR}/downward",
                  "main",
                  ['--search',  "astar(ff)"],
                  build_options = ["-j1"],
                  driver_options = ["--search-time-limit", f"{SEARCH_TIME_LIMIT}", "--overall-time-limit", f"{OVERALL_TIME_LIMIT}", ])

exp.add_suite(BENCHMARKS_DIR, SUITE)

exp.add_parser(exp.EXITCODE_PARSER)
exp.add_parser(exp.TRANSLATOR_PARSER)
exp.add_parser(exp.SINGLE_SEARCH_PARSER)
exp.add_parser(exp.PLANNER_PARSER)

exp.add_step('build', exp.build)
exp.add_step('start', exp.start_runs)
exp.add_fetcher(name='fetch')

exp.add_absolute_report_step(attributes=exp.DEFAULT_TABLE_ATTRIBUTES)

exp.run_steps()
