#! /usr/bin/env python3

import itertools
import math
import os
import subprocess

from lab.environments import LocalEnvironment, BaselSlurmEnvironment
from lab.reports import Attribute, geometric_mean, arithmetic_mean

from downward.reports.compare import ComparativeReport
from tex_scatter import ScatterTexReport
import tex_scatter
import common_setup
from common_setup import IssueConfig, IssueExperiment

DIR = os.path.dirname(os.path.abspath(__file__))
SCRIPT_NAME = os.path.splitext(os.path.basename(__file__))[0]
BENCHMARKS_DIR = f"{DIR}/../pddl/2022-04-07"
PLANNERS_DIR = f"{DIR}/../planners/"
ENVIRONMENT = BaselSlurmEnvironment(partition="rome",email="bx56lg@cs.aau.dk")

if common_setup.is_test_run():
    ENVIRONMENT = LocalEnvironment(processes=4)

exp = IssueExperiment(
    environment=ENVIRONMENT,
)

exp.add_fetcher(name='fetch-1', src='data/exp-2022-04-07a-emoplan-full')
exp.add_fetcher(name='fetch-2', src='data/exp-2022-04-07b-emoplan-partial')
exp.add_fetcher(name='fetch-3', src='data/exp-2022-04-07c-emoplan-full-feasible')
# exp.add_fetcher(name='fetch-4', src='data/exp-2022-04-07d-emoplan-full-unit')
exp.add_fetcher(name='fetch-5', src='data/exp-2022-04-07e-emoplan-partial-feasible')
exp.add_absolute_report_step(attributes=exp.DEFAULT_TABLE_ATTRIBUTES)


def filter_cmp_feasible(run):
    if 'unit' in run['domain']:
        return False

    if 'feasible' in run['domain']:
        run['algorithm'] += '-feasible'

    run['domain'] = run['domain'].replace('-feasible', '')

    run['id'][0] = run['algorithm']
    run['id'][1] = run['domain']

    return run

def filter_rnm(run):
    run['domain'] = run['domain'].replace('emoplan-', '').replace('control', '')
    if 'total_time' in run:
        run['full_time'] = run['total_time'] + run['translator_time_done']
    return run



exp.add_report(
    ScatterTexReport(
        attributes=["cost"], filter = [filter_rnm],
        filter_algorithm=['ff','lmcut'],filter_domain=['full', 'partial'],
        title = "Plan cost on commonly solved instances", show_missing=False,
        xlabel="\\ff",
        ylabel="\\lmcut",
        preamble='\\input{commands}'
    ),name=f"cost-ff-vs-lmcut",
)

# exp.add_report(
#     ScatterTexReport(
#         attributes=['search_time'], filter = [filter_rnm],
#         title = "Search time after grounding (s)",
#         filter_algorithm=['ff','lmcut'],filter_domain=['full', 'partial'],
#         #get_category=lambda run1, run2: run1["domain"]
#         extra_axis_options = {
#             # 'x ticks' : '',
#         },
#     ),name=f"search_time-ff-vs-lmcut",
# )

exp.add_report(
    ScatterTexReport(
        attributes=['total_time'], filter = [filter_rnm],
        title = "Planning time excluding grounding (s)",
        filter_algorithm=['ff','lmcut'],filter_domain=['full', 'partial'],
        manual_plots = [tex_scatter.plots['x10'], tex_scatter.plots['x100']]
    ),name=f"total_time-ff-vs-lmcut",
)


exp.add_report(
    ScatterTexReport(
        attributes=['planner_time'], filter = [filter_rnm],
        title = "Total planning time (s)",
        filter_algorithm=['ff','lmcut'],filter_domain=['full', 'partial'],
        manual_plots = [tex_scatter.plots['x10'], tex_scatter.plots['x100']]
    ),name=f"planner_time-ff-vs-lmcut",
)

exp.add_report(
    ScatterTexReport(
        attributes=['cost'], filter = [filter_rnm,filter_cmp_feasible],
        filter_algorithm=['ff', 'ff-feasible'],
        title = "Plan cost on commonly solved instances", show_missing=False,
        xlabel = "\\ff",
        ylabel = "\\ff feasible",
        preamble='\\input{commands}',
    ) ,name=f"cost-ff-cmp-feasible",
)

exp.add_report(
    ScatterTexReport(
        attributes=['total_time'],
        filter=[filter_rnm, filter_cmp_feasible], filter_algorithm=['ff', 'ff-feasible'],
        title = "Planning time excluding grounding (s)",show_missing=True,
        xlabel = "\\ff",
        ylabel = "\\ff feasible",
        preamble='\\input{commands}',
        manual_plots = [tex_scatter.plots['/10'], tex_scatter.plots['/100']]
    ),name=f"total_time-ff-cmp-feasible",
)


exp.add_report(
    ScatterTexReport(
        attributes=['planner_time'],
        filter=[filter_rnm, filter_cmp_feasible], filter_algorithm=['ff', 'ff-feasible'],
        title = "Total planning time (s)", show_missing=True,
        xlabel = "\\ff",
        ylabel = "\\ff feasible",
        preamble='\\input{commands}',
        manual_plots = [tex_scatter.plots['/10'], tex_scatter.plots['/100']]
    ),name=f"planner_time-ff-cmp-feasible",
)

exp.add_report(
    ScatterTexReport(
        attributes=['cost'],
        filter=[filter_rnm, filter_cmp_feasible], filter_algorithm=['lmcut', 'lmcut-feasible'],
        title = "Plan cost on commonly solved instances", show_missing=False,
        xlabel = "\\lmcut",
        ylabel = "\\lmcut feasible",
        preamble='\\input{commands}',
    ) ,name=f"cost-lmcut-cmp-feasible",
)

exp.add_report(
    ScatterTexReport(
        attributes=['total_time'],
        filter=[filter_rnm, filter_cmp_feasible], filter_algorithm=['lmcut', 'lmcut-feasible'],
        title = "Planning time excluding grounding (s)",show_missing=True,
        xlabel = "\\lmcut",
        ylabel = "\\lmcut feasible",
        preamble='\\input{commands}',
        manual_plots = [tex_scatter.plots['/10'], tex_scatter.plots['/100']]
    ),name=f"total_time-lmcut-cmp-feasible",
)


exp.add_report(
    ScatterTexReport(
        attributes=['planner_time'],
        filter=[filter_rnm, filter_cmp_feasible], filter_algorithm=['lmcut', 'lmcut-feasible'],
        title = "Total planning time (s)", show_missing=True,
        xlabel = "\\lmcut",
        ylabel = "\\lmcut feasible",
        preamble='\\input{commands}',
        manual_plots = [tex_scatter.plots['/10'], tex_scatter.plots['/100']]
    ),name=f"planner_time-lmcut-cmp-feasible",
)


exp.add_report(
    ScatterTexReport(
        attributes=['translator_operators'],
        filter=[filter_rnm, filter_cmp_feasible], filter_algorithm=['ff', 'ff-feasible'],
        get_category=lambda run1, run2: run1["domain"],
        xlabel='Total', ylabel='Feasible', title='Grounded Actions', add_lower_for = False, show_missing=False,
        format_category = lambda x : f"\\{x}control",
        legend_pos = 'north west',
        preamble='\\input{commands}',
        manual_plots = [tex_scatter.plots['/10'], tex_scatter.plots['/100']]
    ) ,name=f"translator_operators-lmcut-cmp-feasible",
)





# for atr in ['cost', 'search_time', 'planner_time','translator_operators', 'initial_h_value']:
#     exp.add_report(
#         ScatterTexReport(
#             attributes=[atr], filter = [filter_rnm],
#             filter_algorithm=['ff','lmcut'],filter_domain=['full', 'partial'],
#             #get_category=lambda run1, run2: run1["domain"]
#         ),name=f"{atr}-ff-vs-lmcut",
#     )

#     exp.add_report(
#         ScatterTexReport(
#             attributes=[atr], filter = [filter_rnm],
#             filter_algorithm=['lmcut', 'scorpion-sys3-30'],filter_domain=['full', 'partial'],
#             #            get_category=lambda run1, run2: run1["domain"] ,
#         ),name=f"{atr}-lmcut-vs-scorpion",
#     )

#     exp.add_report(
#         ScatterTexReport(
#             attributes=[atr], filter = [filter_rnm,filter_cmp_feasible],
#             filter_algorithm=['ff', 'ff-feasible'],
# #            get_category=lambda run1, run2: run1["domain"] ,
#        ) ,name=f"{atr}-ff-cmp-feasible",
#     )


#     exp.add_report(
#         ScatterTexReport(
#             attributes=[atr],
#             filter=[filter_rnm, filter_cmp_feasible], filter_algorithm=['lmcut', 'lmcut-feasible'],
# #            get_category=lambda run1, run2: run1["domain"] ,
#         ) ,name=f"{atr}-lmcut-cmp-feasible",
#     )

# # for atr in ['cost', 'search_time']:
# #     exp.add_report(
# #         ScatterTexReport(
# #             attributes=[atr],
# #             filter_algorithm=['ff','lmcut'],filter_domain=['emoplan-fullcontrol-feasible', 'emoplan-partialcontrol-feasible'],
# #             get_category=lambda run1, run2: run1["domain"] ,
# #             format="tex",
# #         ),name=f"{atr}-ff-vs-lmcut-feasible",
# #     )



exp.run_steps()
