import logging
import os.path

from lab import tools


from collections import defaultdict
import itertools
import logging
import math
import os

from downward.reports import PlanningReport
from lab import tools



plots = {'x10' : r'\addplot[color=black!70,dashed] coordinates {(0.00001, 0.0001) (1000000, 10000000)};',
         'x100' : r'\addplot[color=black!40,dashed] coordinates {(0.00001, 0.001) (100000, 10000000)};',
         '/10' : r'\addplot[color=black!70,dashed] coordinates {(0.0001, 0.00001) (10000000, 1000000)};',
         '/100':  r'\addplot[color=black!40,dashed] coordinates {(0.001, 0.00001) (10000000, 100000)};'
}





class MyScatterPgfplots:
    @classmethod
    def _get_plot(cls, report):
        lines = []
        options = cls._get_axis_options(report)

        if report.extra_axis_options:
            options.update(report.extra_axis_options)

        if report.x_upper is not None:
            options["xmax"] = report.x_upper*report.axis_slack
        if report.y_upper is not None:
            options["ymax"] = report.y_upper*report.axis_slack

        if report.x_lower is not None:
            options["xmin"] = report.x_lower/report.axis_slack
        if report.y_lower is not None:
            options["ymin"] = report.y_lower/report.axis_slack


        lines.append("\\begin{axis}[%s]" % cls._format_options(options))
        for category, coords in sorted(report.categories.items()):
            plot = {"only marks": True}
            lines.append(
                "\\addplot+[{}] coordinates {{\n{}\n}};".format(
                    cls._format_options(plot), " ".join(str(c) for c in coords)
                )
            )
            if category:
                lines.append("\\addlegendentry{%s}" % report.format_category(category))
            elif report.has_multiple_categories():
                # None is treated as the default category if using multiple
                # categories. Add a corresponding entry to the legend.
                lines.append("\\addlegendentry{default}")

        if report.plot_horizontal_line:
            # Add black line at y=1.
            line_min, line_max = cls._get_supported_range(options["xmode"])
            lines.append(
                "\\draw[color=black] (axis cs:{line_min},1) -- "
                "(axis cs:{line_max},1);".format(**locals())
            )
        if report.plot_diagonal_line:
            # Add black diagonal line.
            assert options["xmode"] == options["ymode"]
            line_min, line_max = cls._get_supported_range(options["xmode"])
            lines.append(
                "\\draw[color=black] (axis cs:{line_min},{line_min}) -- "
                "(axis cs:{line_max},{line_max});".format(**locals())
            )

        if report.manual_plots:
            lines += report.manual_plots

        lines.append("\\end{axis}")
        return lines

    @classmethod
    def _get_supported_range(cls, mode):
        # These are approximate values found by trial and error.
        if mode == "normal":
            return "-1e3", "1e3"
        else:
            assert mode == "log"
            return "1e-70", "1e70"

    @classmethod
    def write(cls, report, filename):
        lines = (
            [x for x in [
                r"\documentclass[tikz]{standalone}",
                r"\usepackage{pgfplots}",
                r"\usepackage{xspace}",
                report.preamble,
                r"\begin{document}",
                r"\begin{tikzpicture}",
            ] if x is not None]
            + cls._get_plot(report)
            + [r"\end{tikzpicture}", r"\end{document}"]
        )
        tools.makedirs(os.path.dirname(filename))
        tools.write_file(filename, "\n".join(lines))
        logging.info("Wrote file://%s" % filename)

    @classmethod
    def _get_axis_options(cls, report):
        axis = {}
        axis["xlabel"] = report.xlabel
        axis["ylabel"] = report.ylabel
        axis["title"] = report.title
        axis["legend cell align"] = "left"

        convert_scale = {"log": "log", "symlog": "log", "linear": "normal"}
        axis["xmode"] = convert_scale[report.xscale]
        axis["ymode"] = convert_scale[report.yscale]

        if report.x_missing:
            axis["extra x ticks"] = report.missing_value
            axis["extra x tick style"] = "grid=major,xticklabel style={rotate=90,anchor=east}"
            axis["extra x tick labels"]="uns."

        if report.y_missing:
            axis["extra y ticks"] = report.missing_value
            axis["extra y tick style"] = "grid=major"
            axis["extra y tick labels"]="uns."

        if report.has_multiple_categories():
            axis["legend style"] = cls._format_options(
                {"legend pos": report.legend_pos}
            )

        return axis

    @classmethod
    def _format_options(cls, options):
        opts = []
        for key, value in sorted(options.items()):
            if value is None or value is False:
                continue
            if isinstance(value, bool) or value is None:
                opts.append(key)
            elif isinstance(value, str):
                if " " in value or "=" in value:
                    value = "{%s}" % value
                opts.append("{}={}".format(key, value.replace("_", "-")))
            else:
                opts.append(f"{key}={value}")
        return ", ".join(opts)

class ScatterTexReport(PlanningReport):
    """
    Generate a scatter plot for an attribute.
    """

    def __init__(
        self,
        relative=False,
        show_missing=True,
        add_lower_for=True,
        get_category=None,
        title=None,
        scale=None,
        xlabel="",
        ylabel="",
        extra_axis_options={},
        preamble = None,
        axis_slack=None,
        force_common_min=True,
        format_category= None,
        legend_pos = "outer north east",
        manual_plots = [],
        manual_missing_value = None
        **kwargs,
    ):

        PlanningReport.__init__(self, format='tex', **kwargs)
        self.relative = relative
        if len(self.attributes) != 1:
            logging.critical("ScatterPlotReport needs exactly one attribute")
        self.attribute = self.attributes[0]
        # By default all values are in the same category "None".
        self.get_category = get_category or (lambda run1, run2: None)
        self.show_missing = show_missing
        self.add_lower_for = add_lower_for
        self.writer = MyScatterPgfplots

        self.title = title if title is not None else (self.attribute or "")
        self._set_scales(scale)
        self.axis_slack = axis_slack or (1.05 if self.xscale == self.yscale == 'linear' else 1.5)
        self.xlabel = xlabel
        self.ylabel = ylabel
        self.preamble = preamble
        self.x_lower = None
        self.y_lower = None
        self.format_category = format_category or (lambda x : x)
        self.legend_pos = legend_pos
        self.extra_axis_options = extra_axis_options
        self.manual_plots = manual_plots
        self.manual_missing_value = manual_missing_value
        # If the size has not been set explicitly, make it a square.
        self.force_common_min= force_common_min

    def _set_scales(self, scale):
        self.xscale = scale or self.attribute.scale or "log"
        self.yscale = "log" if self.relative else self.xscale
        scales = ["linear", "log", "symlog"]
        for scale in [self.xscale, self.yscale]:
            if scale not in scales:
                logging.critical(f"Scale {scale} not in {scales}")

    def has_multiple_categories(self):
        return any(key is not None for key in self.categories.keys())

    def count_lower_for(self):
        res = defaultdict(int)
        for coords in self.categories.values():
            for x, y in coords:
                if x is not None:
                    if y is None:
                        res ['only_x'] += 1
                    elif x < y:
                        res ['lower_x'] += 1
                    elif y < x:
                        res ['lower_y'] += 1

                elif y is not None:
                    res ['only_y'] += 1
        return res

    def common_max(self):
        return max ([max(x, y) for coords in self.categories.values() for x, y in coords  ])

    def common_min(self):
        return min ([min(x, y) for coords in self.categories.values() for x, y in coords  ])

    def _fill_categories(self):
        """Map category names to coordinate lists."""
        categories = defaultdict(list)
        for runs in self.problem_runs.values():
            try:
                run1, run2 = runs
            except ValueError:
                logging.critical(
                    "Scatter plot needs exactly two runs for {domain}:{problem}. "
                    "Instead of filtering a whole run, try setting only some of its "
                    "attribute values to None in a filter.".format(**runs[0])
                )
            category = self.get_category(run1, run2)
            coord = (run1.get(self.attribute), run2.get(self.attribute))
            if self.show_missing or None not in coord:
                categories[category].append(coord)
        return categories

    def _turn_into_relative_coords(self, categories):
        assert self.relative
        y_rel_max = 0
        for coords in categories.values():
            for x, y in coords:
                if (x is not None and x <= 0) or (y is not None and y <= 0):
                    logging.critical("Relative scatter plots need values > 0.")
                if x is not None and y is not None:
                    y_rel_max = max(y_rel_max, y / float(x))
        y_rel_missing = y_rel_max * 1.5 if y_rel_max != 0 else None
        x_missing = self._compute_missing_value(categories, 0, self.xscale)
        self.x_upper = x_missing
        self.y_upper = y_rel_missing

        new_categories = {}
        for category, coords in categories.items():
            new_coords = []
            for coord in coords:
                x, y = coord
                if x is None and y is None:
                    x, y = x_missing, y_rel_missing
                elif x is None and y is not None:
                    x, y = x_missing, 1
                elif x is not None and y is None:
                    x, y = x, y_rel_missing
                elif x is not None and y is not None:
                    x, y = x, y / float(x)
                new_coords.append((x, y))
            if new_coords:
                new_categories[category] = new_coords
        return new_categories

    def _compute_missing_value(self, categories, axis, scale):
        if not self.show_missing:
            return None
        if not any(
            coord[axis] is None for coords in categories.values() for coord in coords
        ):
            return None
        max_value = max(
            coord[axis]
            for coords in categories.values()
            for coord in coords
            if coord[axis] is not None
        )
        if max_value is None:
            return 1
        if scale == "linear":
            return max_value * 1.1
        return 2*int(10 ** math.ceil(math.log10(max_value)))

    def _handle_non_positive_values(self, categories):
        """Plot integer 0 values at 0.1 in log plots and abort if any value is < 0."""
        assert not self.relative
        assert self.xscale == self.yscale == "log"
        new_categories = {}
        for category, coords in categories.items():
            new_coords = []
            for x, y in coords:
                if x == 0 and isinstance(x, int):
                    x = 0.1
                if y == 0 and isinstance(y, int):
                    y = 0.1

                if (x is not None and x <= 0) or (y is not None and y <= 0):
                    logging.critical(
                        "Logarithmic axes can only show positive values. "
                        "Use a symlog or linear scale instead."
                    )
                else:
                    new_coords.append((x, y))
            new_categories[category] = new_coords
        return new_categories

    def _handle_missing_values(self, categories):
        assert not self.relative
        self.x_missing = self._compute_missing_value(categories, 0, self.xscale)
        self.y_missing = self._compute_missing_value(categories, 1, self.yscale)
        if self.x_missing is None:
            missing_value = self.y_missing
        elif self.y_missing is None:
            missing_value = self.x_missing
        else:
            missing_value = max(self.x_missing, self.y_missing)

        if self.manual_missing_value:
            missing_value = manual_missing_value

        self.missing_value = missing_value

        if missing_value is None:
            self.x_upper = self.common_max()
            self.y_upper = self.common_max()
        else:
            self.x_upper = missing_value
            self.y_upper = missing_value

        if not self.show_missing:
            # Coords with None values have already been filtered.
            return categories

        new_categories = {}
        for category, coords in categories.items():
            coords = [
                (
                    x if x is not None else missing_value,
                    y if y is not None else missing_value,
                )
                for x, y in coords
            ]
            if coords:
                new_categories[category] = coords
        return new_categories

    def _get_category_styles(self, categories):
        """
        Create dictionary mapping from category name to marker style.
        """
        shapes = "x+os^v<>D"
        colors = [f"C{c}" for c in range(10)]

        num_styles = len(shapes) * len(colors)
        styles = [
            {"marker": shape, "c": color}
            for shape, color in itertools.islice(
                zip(itertools.cycle(shapes), itertools.cycle(colors)), num_styles
            )
        ]
        assert (
            len({(s["marker"], s["c"]) for s in styles}) == num_styles
        ), "The number of shapes and the number of colors must be coprime."

        category_styles = {}
        for i, category in enumerate(sorted(categories)):
            category_styles[category] = styles[i % len(styles)]
        return category_styles

    def _write_plot(self, runs, filename):
        # Map category names to coord tuples.
        self.categories = self._fill_categories()
        if self.relative:
            self.plot_diagonal_line = False
            self.plot_horizontal_line = True
            self.categories = self._turn_into_relative_coords(self.categories)
        else:
            self.plot_diagonal_line = True
            self.plot_horizontal_line = False
            if self.xscale == "log":
                assert self.yscale == "log"
                self.categories = self._handle_non_positive_values(self.categories)
            self.categories = self._handle_missing_values(self.categories)
        if not self.categories:
            logging.critical("Plot contains no points.")

        if self.add_lower_for:
            res = self.count_lower_for()
            self.xlabel += f" (lower for {res['lower_x']} tasks)"
            self.ylabel += f" (lower for {res['lower_y']} tasks)"

        if self.force_common_min:
            self.x_lower = self.common_min()
            self.y_lower = self.x_lower


        self.styles = self._get_category_styles(self.categories)
        self.writer.write(self, filename)

    def write(self):
        if not len(self.algorithms) == 2:
            logging.critical(
                "Scatter plots need exactly 2 algorithms: %s" % self.algorithms
            )
        self.xlabel = self.xlabel or self.algorithms[0]
        self.ylabel = self.ylabel or self.algorithms[1]



        suffix = ".tex"
        if not self.outfile.endswith(suffix):
            self.outfile += suffix
        tools.makedirs(os.path.dirname(self.outfile))
        self._write_plot(self.runs.values(), self.outfile)
